@@ -16,13 +16,16 @@ case $PYTHON_VERSION in
   INSTALLER_NAME=python-$FULL_VERSION-macosx10.9.pkg
   ;;
 3.10)
-  FULL_VERSION=3.10.4
+  FULL_VERSION=3.10.11
   INSTALLER_NAME=python-$FULL_VERSION-macos11.pkg
   ;;
 3.11)
-  FULL_VERSION=3.11.0
+  FULL_VERSION=3.11.7
   INSTALLER_NAME=python-$FULL_VERSION-macos11.pkg
   ;;
+3.12)
+  FULL_VERSION=3.12.0
+  INSTALLER_NAME=python-$FULL_VERSION-macos11.pkg
 esac
 
 URL=https://www.python.org/ftp/python/$FULL_VERSION/$INSTALLER_NAME
@@ -35,7 +38,7 @@ curl $URL > $INSTALLER_NAME
 
 sudo installer -pkg $INSTALLER_NAME -target /
 
-sudo rm /usr/local/bin/python
+sudo rm -f /usr/local/bin/python
 sudo ln -s /usr/local/bin/python$PYTHON_VERSION /usr/local/bin/python
 
 which python
