@@ -20,6 +20,7 @@ from ._jinit import *
 from ._jpackage import *
 from ._jproxy import *
 from ._core import *
+from . import _core
 from ._gui import *
 from ._classpath import *
 from ._jclass import *
@@ -41,17 +42,17 @@ from . import protocol      # lgtm [py/import-own-module]
 from . import _jthread      # lgtm [py/import-own-module]
 
 __all__ = ['java', 'javax']
-__all__.extend(_jinit.__all__)
+__all__.extend(_jinit.__all__)  # type: ignore[name-defined]
 __all__.extend(_core.__all__)
-__all__.extend(_classpath.__all__)
-__all__.extend(types.__all__)
-__all__.extend(_jproxy.__all__)
-__all__.extend(_jpackage.__all__)
-__all__.extend(_jclass.__all__)
-__all__.extend(_jcustomizer.__all__)
-__all__.extend(_gui.__all__)
+__all__.extend(_classpath.__all__)  # type: ignore[name-defined]
+__all__.extend(types.__all__)  # type: ignore[name-defined]
+__all__.extend(_jproxy.__all__)  # type: ignore[name-defined]
+__all__.extend(_jpackage.__all__)  # type: ignore[name-defined]
+__all__.extend(_jclass.__all__)  # type: ignore[name-defined]
+__all__.extend(_jcustomizer.__all__)  # type: ignore[name-defined]
+__all__.extend(_gui.__all__)  # type: ignore[name-defined]
 
-__version__ = "1.4.1"
+__version__ = "1.5.0"
 __version_info__ = __version__.split('.')
 
 
@@ -65,5 +66,5 @@ JMethod = _jpype._JMethod
 JField = _jpype._JField
 
 if hasattr(_jpype, 'bootstrap'):
-   _jpype.bootstrap()
-   _core.initializeResources()
+    _jpype.bootstrap()
+    _core.initializeResources()
