@@ -26,7 +26,7 @@ class JPBufferType;
 class JPBuffer
 {
 public:
-	JPBuffer(const JPValue& array);
+	explicit JPBuffer(const JPValue& array);
 	virtual~ JPBuffer();
 
 	JPBufferType* getClass()
@@ -39,19 +39,19 @@ public:
 		return m_Object.get();
 	}
 
-	bool isReadOnly();
+	bool isReadOnly() const;
 
 	Py_buffer& getView();
 
-	bool isValid();
+	bool isValid() const;
 
 private:
 	JPBufferType* m_Class;
 	JPObjectRef m_Object;
 	void *m_Address;
 	Py_ssize_t m_Capacity;
-	Py_buffer m_Buffer;
-	char m_Format[3];
+	Py_buffer m_Buffer{};
+	char m_Format[3]{};
 } ;
 
 #endif // _JPBUFFER_H_
\ No newline at end of file
