@@ -23,14 +23,14 @@ class JPBufferType : public JPClass
 {
 public:
 	JPBufferType(JPJavaFrame& frame, jclass cls, const string& name, JPClass* superClass, const JPClassList& interfaces, jint modifiers);
-	virtual~ JPBufferType();
+	~ JPBufferType() override;
 
-	char* getType()
+	const char* getType()
 	{
-		return const_cast<char*> (m_Type);
+		return m_Type;
 	}
 
-	int getSize()
+	int getSize() const
 	{
 		return m_Size;
 	}
