@@ -33,7 +33,7 @@ public:
 
 	JPRef()
 	{
-		m_Context = 0;
+		m_Context = nullptr;
 		m_Ref = 0;
 	}
 
@@ -41,7 +41,7 @@ public:
 	{
 		m_Context = context;
 		m_Ref = 0;
-		if (context == 0)
+		if (context == nullptr)
 			return;
 		JPJavaFrame frame = JPJavaFrame::outer(m_Context);
 		m_Ref = (jref) frame.NewGlobalRef((jobject) obj);
@@ -67,10 +67,10 @@ public:
 	}
 } ;
 
-typedef JPRef<jclass> JPClassRef;
-typedef JPRef<jobject> JPObjectRef;
-typedef JPRef<jarray> JPArrayRef;
-typedef JPRef<jthrowable> JPThrowableRef;
+using JPClassRef = JPRef<jclass>;
+using JPObjectRef = JPRef<jobject>;
+using JPArrayRef = JPRef<jarray>;
+using JPThrowableRef = JPRef<jthrowable>;
 
 class JPStackInfo;
 class JPGarbageCollection;
@@ -120,8 +120,9 @@ public:
 
 	JPContext();
 	virtual ~JPContext();
+    JPContext(const JPContext& orig) = delete;
 
-	// JVM control functions
+    // JVM control functions
 	bool isRunning();
 	void startJVM(const string& vmPath, const StringVector& args,
 			bool ignoreUnrecognized, bool convertStrings, bool interrupt);
@@ -170,52 +171,51 @@ public:
 	}
 
 	// Java type resources
-	JPPrimitiveType* _void;
-	JPPrimitiveType* _boolean;
-	JPPrimitiveType* _byte;
-	JPPrimitiveType* _char;
-	JPPrimitiveType* _short;
-	JPPrimitiveType* _int;
-	JPPrimitiveType* _long;
-	JPPrimitiveType* _float;
-	JPPrimitiveType* _double;
+	JPPrimitiveType* _void{};
+	JPPrimitiveType* _boolean{};
+	JPPrimitiveType* _byte{};
+	JPPrimitiveType* _char{};
+	JPPrimitiveType* _short{};
+	JPPrimitiveType* _int{};
+	JPPrimitiveType* _long{};
+	JPPrimitiveType* _float{};
+	JPPrimitiveType* _double{};
 
-	JPBoxedType* _java_lang_Void;
-	JPBoxedType* _java_lang_Boolean;
-	JPBoxedType* _java_lang_Byte;
-	JPBoxedType* _java_lang_Character;
-	JPBoxedType* _java_lang_Short;
-	JPBoxedType* _java_lang_Integer;
-	JPBoxedType* _java_lang_Long;
-	JPBoxedType* _java_lang_Float;
-	JPBoxedType* _java_lang_Double;
+	JPBoxedType* _java_lang_Void{};
+	JPBoxedType* _java_lang_Boolean{};
+	JPBoxedType* _java_lang_Byte{};
+	JPBoxedType* _java_lang_Character{};
+	JPBoxedType* _java_lang_Short{};
+	JPBoxedType* _java_lang_Integer{};
+	JPBoxedType* _java_lang_Long{};
+	JPBoxedType* _java_lang_Float{};
+	JPBoxedType* _java_lang_Double{};
 
-	JPClass* _java_lang_Object;
-	JPClass* _java_lang_Class;
-	JPClass* _java_lang_reflect_Field;
-	JPClass* _java_lang_reflect_Method;
-	JPClass* _java_lang_Throwable;
-	JPStringType* _java_lang_String;
-	JPClass* _java_nio_ByteBuffer;
+	JPClass* _java_lang_Object{};
+	JPClass* _java_lang_Class{};
+	JPClass* _java_lang_reflect_Field{};
+	JPClass* _java_lang_reflect_Method{};
+	JPClass* _java_lang_Throwable{};
+	JPStringType* _java_lang_String{};
+	JPClass* _java_nio_ByteBuffer{};
 
 private:
 
 	void loadEntryPoints(const string& path);
 
-	jint(JNICALL * CreateJVM_Method)(JavaVM **pvm, void **penv, void *args);
-	jint(JNICALL * GetCreatedJVMs_Method)(JavaVM **pvm, jsize size, jsize * nVms);
+	jint(JNICALL * CreateJVM_Method)(JavaVM **pvm, void **penv, void *args){};
+	jint(JNICALL * GetCreatedJVMs_Method)(JavaVM **pvm, jsize size, jsize * nVms){};
 
 private:
-	JPContext(const JPContext& orig);
 
-	JavaVM *m_JavaVM;
+	JavaVM *m_JavaVM{};
 
 	// Java half
 	JPObjectRef m_JavaContext;
 
 	// Services
-	JPTypeManager *m_TypeManager;
-	JPClassLoader *m_ClassLoader;
+	JPTypeManager *m_TypeManager{};
+	JPClassLoader *m_ClassLoader{};
 
 public:
 	JPClassRef m_ContextClass;
@@ -226,43 +226,43 @@ private:
 	JPClassRef m_Array;
 
 	// Java Functions
-	jmethodID m_Object_ToStringID;
-	jmethodID m_Object_EqualsID;
-	jmethodID m_Object_HashCodeID;
-	jmethodID m_CallMethodID;
-	jmethodID m_Class_GetNameID;
-	jmethodID m_Context_collectRectangularID;
-	jmethodID m_Context_assembleID;
-	jmethodID m_String_ToCharArrayID;
-	jmethodID m_Context_CreateExceptionID;
-	jmethodID m_Context_GetExcClassID;
-	jmethodID m_Context_GetExcValueID;
-	jmethodID m_Context_ClearInterruptID;
-	jmethodID m_CompareToID;
-	jmethodID m_Buffer_IsReadOnlyID;
-	jmethodID m_Context_OrderID;
-	jmethodID m_Object_GetClassID;
-	jmethodID m_Array_NewInstanceID;
-	jmethodID m_Throwable_GetCauseID;
-	jmethodID m_Throwable_GetMessageID;
-	jmethodID m_Context_GetFunctionalID;
+	jmethodID m_Object_ToStringID{};
+	jmethodID m_Object_EqualsID{};
+	jmethodID m_Object_HashCodeID{};
+	jmethodID m_CallMethodID{};
+	jmethodID m_Class_GetNameID{};
+	jmethodID m_Context_collectRectangularID{};
+	jmethodID m_Context_assembleID{};
+	jmethodID m_String_ToCharArrayID{};
+	jmethodID m_Context_CreateExceptionID{};
+	jmethodID m_Context_GetExcClassID{};
+	jmethodID m_Context_GetExcValueID{};
+	jmethodID m_Context_ClearInterruptID{};
+	jmethodID m_CompareToID{};
+	jmethodID m_Buffer_IsReadOnlyID{};
+	jmethodID m_Context_OrderID{};
+	jmethodID m_Object_GetClassID{};
+	jmethodID m_Array_NewInstanceID{};
+	jmethodID m_Throwable_GetCauseID{};
+	jmethodID m_Throwable_GetMessageID{};
+	jmethodID m_Context_GetFunctionalID{};
 	friend class JPProxy;
 	JPClassRef m_ProxyClass;
-	jmethodID m_Proxy_NewID;
-	jmethodID m_Proxy_NewInstanceID;
+	jmethodID m_Proxy_NewID{};
+	jmethodID m_Proxy_NewInstanceID{};
 
-	jmethodID m_Context_IsPackageID;
-	jmethodID m_Context_GetPackageID;
-	jmethodID m_Package_GetObjectID;
-	jmethodID m_Package_GetContentsID;
-	jmethodID m_Context_NewWrapperID;
+	jmethodID m_Context_IsPackageID{};
+	jmethodID m_Context_GetPackageID{};
+	jmethodID m_Package_GetObjectID{};
+	jmethodID m_Package_GetContentsID{};
+	jmethodID m_Context_NewWrapperID{};
 public:
-	jmethodID m_Context_GetStackFrameID;
+	jmethodID m_Context_GetStackFrameID{};
 	void onShutdown();
 
 private:
-	bool m_Running;
-	bool m_ConvertStrings;
+	bool m_Running{};
+	bool m_ConvertStrings{};
 	bool m_Embedded;
 public:
 	JPGarbageCollection *m_GC;
@@ -280,7 +280,7 @@ template<class jref>
 JPRef<jref>::JPRef(const JPRef& other)
 {
 	m_Context = other.m_Context;
-	if (m_Context != NULL)
+	if (m_Context != nullptr)
 	{
 		JPJavaFrame frame = JPJavaFrame::external(m_Context, m_Context->getEnv());
 		m_Ref = (jref) frame.NewGlobalRef((jobject) other.m_Ref);
@@ -294,7 +294,7 @@ JPRef<jref>::JPRef(const JPRef& other)
 template<class jref>
 JPRef<jref>::~JPRef()
 {
-	if (m_Ref != 0 && m_Context != 0)
+	if (m_Ref != 0 && m_Context != nullptr)
 	{
 		m_Context->ReleaseGlobalRef((jobject) m_Ref);
 	}
@@ -307,7 +307,7 @@ JPRef<jref>& JPRef<jref>::operator=(const JPRef<jref>& other)
 		return *this;
 	// m_Context may or may not be set up here, so we need to use a
 	// different frame for unreferencing and referencing
-	if (m_Context != 0 && m_Ref != 0)
+	if (m_Context != nullptr && m_Ref != 0)
 	{  // GCOVR_EXCL_START
 		// This code is not currently used.
 		JPJavaFrame frame = JPJavaFrame::external(m_Context, m_Context->getEnv());
@@ -316,7 +316,7 @@ JPRef<jref>& JPRef<jref>::operator=(const JPRef<jref>& other)
 	}  // GCOVR_EXCL_STOP
 	m_Context = other.m_Context;
 	m_Ref = other.m_Ref;
-	if (m_Context != 0 && m_Ref != 0)
+	if (m_Context != nullptr && m_Ref != 0)
 	{
 		JPJavaFrame frame = JPJavaFrame::external(m_Context, m_Context->getEnv());
 		m_Ref = (jref) frame.NewGlobalRef((jobject) m_Ref);
