@@ -27,17 +27,18 @@ public:
 		_none = 0,
 		_explicit = 1,
 		_implicit = 2,
-		_exact = 3
+		_derived = 3,
+		_exact = 4
 	} ;
 
 public:
 	JPMatch();
 	JPMatch(JPJavaFrame *frame, PyObject *object);
 
-	JPContext *getContext()
+	JPContext *getContext() const
 	{
-		if (frame == NULL)
-			return NULL;
+		if (frame == nullptr)
+			return nullptr;
 		return frame->getContext();
 	}
 
@@ -91,4 +92,4 @@ public:
 	char m_Skip;
 } ;
 
-#endif /* JP_MATCH_H */
\ No newline at end of file
+#endif /* JP_MATCH_H */
