@@ -1,30 +1,38 @@
 /*****************************************************************************
-   Copyright 2004 Steve Mnard
-
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
 
-       http://www.apache.org/licenses/LICENSE-2.0
+		http://www.apache.org/licenses/LICENSE-2.0
 
    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
-   
-*****************************************************************************/   
+
+   See NOTICE file for details.
+ *****************************************************************************/
 #ifndef _JPMONITOR_H_
 #define _JPMONITOR_H_
 
 class JPMonitor
 {
-public :
-	JPMonitor(jobject obj);
+public:
+	JPMonitor(JPContext* context, jobject obj);
 	virtual ~JPMonitor();
-	
-private :
-	jobject m_Object;
-};
 
-#endif // _JPMONITOR_H_
+	void enter();
+	void exit();
+
+	JPContext* getContext()
+	{
+		return m_Context;
+	}
+
+private:
+	JPContext* m_Context;
+	JPObjectRef m_Value;
+} ;
+
+#endif // _JPMONITOR_H_
\ No newline at end of file
