@@ -20,11 +20,11 @@
 class JPPrimitiveType : public JPClass
 {
 protected:
-	JPPrimitiveType(const string& name);
-	virtual ~JPPrimitiveType();
+	explicit JPPrimitiveType(const string& name);
+	~JPPrimitiveType() override;
 
 public:
-	virtual bool isPrimitive() const override;
+	bool isPrimitive() const override;
 
 	virtual JPClass* getBoxedClass(JPContext *context) const = 0;
 
