@@ -29,15 +29,14 @@ JPFunctional::JPFunctional(JPJavaFrame& frame, jclass clss,
 }
 
 JPFunctional::~JPFunctional()
-{
-}
+= default;
 
 
 class JPConversionFunctional : public JPConversion
 {
 public:
 
-	virtual JPMatch::Type matches(JPClass *cls, JPMatch &match) override
+	JPMatch::Type matches(JPClass *cls, JPMatch &match) override
 	{
 		if (!PyCallable_Check(match.object))
 			return match.type = JPMatch::_none;
@@ -55,10 +54,10 @@ public:
 		if (PyFunction_Check(match.object))
 		{
 			PyObject* func = match.object; 
-			PyCodeObject* code = (PyCodeObject*) PyFunction_GetCode(func); // borrowed
+			auto* code = (PyCodeObject*) PyFunction_GetCode(func); // borrowed
 			Py_ssize_t args = code->co_argcount;
 			bool is_varargs = ((code->co_flags&CO_VARARGS)==CO_VARARGS);
-			int optional = 0;
+			Py_ssize_t optional = 0;
 			JPPyObject defaults = JPPyObject::accept(PyObject_GetAttrString(func, "__defaults__"));
 			if (!defaults.isNull() && defaults.get() != Py_None)
 				optional = PyTuple_Size(defaults.get());
@@ -73,16 +72,16 @@ public:
 		else if (PyMethod_Check(match.object))
 		{
 			PyObject* func = PyMethod_Function(match.object); // borrowed
-			PyCodeObject* code = (PyCodeObject*) PyFunction_GetCode(func); // borrowed
+			auto* code = (PyCodeObject*) PyFunction_GetCode(func); // borrowed
 			Py_ssize_t args = code->co_argcount;
 			bool is_varargs = ((code->co_flags&CO_VARARGS)==CO_VARARGS);
-			int optional = 0;
+            Py_ssize_t optional = 0;
 			JPPyObject defaults = JPPyObject::accept(PyObject_GetAttrString(func, "__defaults__"));
 			if (!defaults.isNull() && defaults.get() != Py_None)
 				optional = PyTuple_Size(defaults.get());
 			const int jargs = cls->getContext()->getTypeManager()->interfaceParameterCount(cls);
 			// Bound self argument removes one argument
-			if ((PyMethod_Self(match.object))!=NULL) // borrowed
+			if ((PyMethod_Self(match.object))!=nullptr) // borrowed
 				args--;
 			// Too few arguments
 			if (!is_varargs && args < jargs)
@@ -96,20 +95,20 @@ public:
 		return match.type = JPMatch::_implicit;
 	}
 
-	virtual void getInfo(JPClass *cls, JPConversionInfo &info) override
+	void getInfo(JPClass *cls, JPConversionInfo &info) override
 	{
 		PyObject *typing = PyImport_AddModule("jpype.protocol");
 		JPPyObject proto = JPPyObject::call(PyObject_GetAttrString(typing, "Callable"));
 		PyList_Append(info.implicit, proto.get());
 	}
 
-	virtual jvalue convert(JPMatch &match) override
+	jvalue convert(JPMatch &match) override
 	{
-		JPFunctional *cls = (JPFunctional*) match.closure;
+		auto *cls = (JPFunctional*) match.closure;
 		JP_TRACE_IN("JPConversionFunctional::convert");
 		JPContext *context = PyJPModule_getContext();
 		JPJavaFrame frame = JPJavaFrame::inner(context);
-		PyJPProxy *self = (PyJPProxy*) PyJPProxy_Type->tp_alloc(PyJPProxy_Type, 0);
+		auto *self = (PyJPProxy*) PyJPProxy_Type->tp_alloc(PyJPProxy_Type, 0);
 		JP_PY_CHECK();
 		JPClassList cl;
 		cl.push_back(cls);
