@@ -28,8 +28,7 @@ JPStringType::JPStringType(JPJavaFrame& frame,
 }
 
 JPStringType::~JPStringType()
-{
-}
+= default;
 
 JPPyObject JPStringType::convertToPythonObject(JPJavaFrame& frame, jvalue val, bool cast)
 {
@@ -39,7 +38,7 @@ JPPyObject JPStringType::convertToPythonObject(JPJavaFrame& frame, jvalue val, b
 	if (!cast)
 	{
 		// This loses type
-		if (val.l == NULL)
+		if (val.l == nullptr)
 		{
 			return JPPyObject::getNone();
 		}
