@@ -23,7 +23,7 @@ extern "C"
 {
 #endif
 
-PyTypeObject *PyJPChar_Type = NULL;
+PyTypeObject *PyJPChar_Type = nullptr;
 
 struct PyJPChar
 {
@@ -54,10 +54,10 @@ static Py_UCS4 ord(PyObject *c)
 
 static int isNull(JPValue *javaSlot)
 {
-	if (javaSlot != NULL )
+	if (javaSlot != nullptr )
 	{
 		JPClass *cls = javaSlot->getClass();
-		if (cls->isPrimitive() || javaSlot->getValue().l != NULL)
+		if (cls->isPrimitive() || javaSlot->getValue().l != nullptr)
 			return 0;
 	}
 	return 1;
@@ -66,7 +66,7 @@ static int isNull(JPValue *javaSlot)
 static PyObject* notSupported()
 {
 	PyErr_SetString(PyExc_TypeError, "unsupported operation");
-	return 0;
+	return nullptr;
 }
 
 static int assertNotNull(JPValue *javaSlot)
@@ -79,46 +79,63 @@ static int assertNotNull(JPValue *javaSlot)
 
 PyObject *PyJPChar_Create(PyTypeObject *type, Py_UCS2 p)
 {
+	// Allocate a new string type (derived from UNICODE)
 	PyJPChar  *self = (PyJPChar*) PyJPValue_alloc(type, 0);
-	if (self == 0)
-		return 0;
+	if (self == nullptr)
+		return nullptr;
+
+	// Set up a wide char with value of zero
 	self->m_Data[0] = 0;
 	self->m_Data[1] = 0;
 	self->m_Data[2] = 0;
 	self->m_Data[3] = 0;
 
+	// Values taken from internal/cpython/unicode.h
+	
+	// Mark the type in unicode
 	_PyUnicode_LENGTH(self) = 1;
 	_PyUnicode_HASH(self) = -1;
-	_PyUnicode_STATE(self).kind = PyUnicode_1BYTE_KIND;
 
-	_PyUnicode_STATE(self).ascii = 0;
-	_PyUnicode_STATE(self).ready = 1;
-	_PyUnicode_STATE(self).interned = 0;
 	_PyUnicode_STATE(self).compact = 1;
+	_PyUnicode_STATE(self).interned = 0;
 
+#if PY_VERSION_HEX < 0x030c0000
+	_PyUnicode_STATE(self).ready = 1;
+#endif
+
+	// Copy the value based on the length
 	if (p < 128)
 	{
 		_PyUnicode_STATE(self).ascii = 1;
+		_PyUnicode_STATE(self).kind = PyUnicode_1BYTE_KIND;
+
 		char *data = (char*) (((PyASCIIObject*) self) + 1);
-		data[0] = p;
+		data[0] = (char) p;
 		data[1] = 0;
-	} else
-		if (p < 256)
+	} else if (p < 256)
 	{
+		_PyUnicode_STATE(self).ascii = 0;
+		_PyUnicode_STATE(self).kind = PyUnicode_1BYTE_KIND;
+
 		char *data = (char*) ( ((PyCompactUnicodeObject*) self) + 1);
-		data[0] = p;
+		data[0] = (char) p;
 		data[1] = 0;
+	
+#if PY_VERSION_HEX < 0x030c0000
 		_PyUnicode_WSTR_LENGTH(self) = 0;
-		_PyUnicode_WSTR(self) = NULL;
+		_PyUnicode_WSTR(self) = nullptr;
+#endif
 		self->m_Obj.utf8 = NULL;
 		self->m_Obj.utf8_length = 0;
 	} else
 	{
+		_PyUnicode_STATE(self).ascii = 0;
+		_PyUnicode_STATE(self).kind = PyUnicode_2BYTE_KIND;
 
-		Py_UCS2 *data = (Py_UCS2*) ( ((PyCompactUnicodeObject*) self) + 1);
+		auto *data = (Py_UCS2*) ( ((PyCompactUnicodeObject*) self) + 1);
 		data[0] = p;
 		data[1] = 0;
-		_PyUnicode_STATE(self).kind = PyUnicode_2BYTE_KIND;
+#if PY_VERSION_HEX < 0x030c0000
 		if (sizeof (wchar_t) == 2)
 		{
 			_PyUnicode_WSTR_LENGTH(self) = 1;
@@ -126,11 +143,13 @@ PyObject *PyJPChar_Create(PyTypeObject *type, Py_UCS2 p)
 		} else
 		{
 			_PyUnicode_WSTR_LENGTH(self) = 0;
-			_PyUnicode_WSTR(self) = NULL;
+			_PyUnicode_WSTR(self) = nullptr;
 		}
-		self->m_Obj.utf8 = NULL;
+#endif
+		self->m_Obj.utf8 = nullptr;
 		self->m_Obj.utf8_length = 0;
 	}
+
 	return (PyObject*) self;
 }
 
@@ -141,8 +160,8 @@ Py_UCS2 fromJPValue(const JPValue & value)
 	JPClass* cls = value.getClass();
 	if (cls->isPrimitive())
 		return (Py_UCS2) (value.getValue().c);
-	JPPrimitiveType* pcls = ((JPBoxedType*) cls)->getPrimitive();
-	if (value.getValue().l == 0)
+	JPPrimitiveType* pcls = (dynamic_cast<JPBoxedType*>( cls))->getPrimitive();
+	if (value.getValue().l == nullptr)
 		return (Py_UCS2) - 1;
 	else
 		return (Py_UCS2) (pcls->getValueFromObject(value).getValue().c);
@@ -154,15 +173,15 @@ Py_UCS2 fromJPChar(PyJPChar *self)
 {
 	if (_PyUnicode_STATE(self).ascii == 1)
 	{
-		Py_UCS1 *data = (Py_UCS1*) (((PyASCIIObject*) self) + 1);
+		auto *data = (Py_UCS1*) (((PyASCIIObject*) self) + 1);
 		return data[0];
 	}
 	if (_PyUnicode_STATE(self).kind == PyUnicode_1BYTE_KIND)
 	{
-		Py_UCS1 *data = (Py_UCS1*) ( ((PyCompactUnicodeObject*) self) + 1);
+		auto *data = (Py_UCS1*) ( ((PyCompactUnicodeObject*) self) + 1);
 		return data[0];
 	}
-	Py_UCS2 *data = (Py_UCS2*) ( ((PyCompactUnicodeObject*) self) + 1);
+	auto *data = (Py_UCS2*) ( ((PyCompactUnicodeObject*) self) + 1);
 	return data[0];
 }
 
@@ -171,10 +190,10 @@ static PyObject * PyJPChar_new(PyTypeObject *type, PyObject *pyargs, PyObject *
 	JP_PY_TRY("PyJPChar_new");
 	// Get the Java class from the type.
 	JPClass *cls = PyJPClass_getJPClass((PyObject*) type);
-	if (cls == NULL)
+	if (cls == nullptr)
 	{  // GCOVR_EXCL_START
 		PyErr_SetString(PyExc_TypeError, "Java class type is incorrect");
-		return 0;
+		return nullptr;
 	}  // GCOVR_EXCL_STOP
 
 	JPContext *context = PyJPModule_getContext();
@@ -184,7 +203,7 @@ static PyObject * PyJPChar_new(PyTypeObject *type, PyObject *pyargs, PyObject *
 	if (PyTuple_Size(pyargs) != 1)
 	{
 		PyErr_SetString(PyExc_TypeError, "Java chars require one argument");
-		return 0;
+		return nullptr;
 	}
 
 	JPValue jv;
@@ -210,14 +229,14 @@ static PyObject * PyJPChar_new(PyTypeObject *type, PyObject *pyargs, PyObject *
 	{
 		// This is not strictly true as we can cast a float to a char
 		PyErr_SetString(PyExc_TypeError, "Java require index or str with length 1");
-		return 0;
+		return nullptr;
 	}
 
 	PyObject *self = PyJPChar_Create(type, fromJPValue(jv));
 	JP_PY_CHECK();
 	PyJPValue_assignJavaSlot(frame, self, jv);
 	return self;
-	JP_PY_CATCH(NULL);  // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr);  // GCOVR_EXCL_LINE
 }
 
 static PyObject *PyJPChar_str(PyJPChar *self)
@@ -225,16 +244,16 @@ static PyObject *PyJPChar_str(PyJPChar *self)
 	JP_PY_TRY("PyJPChar_str");
 	PyJPModule_getContext(); // Check that JVM is running
 	JPValue *javaSlot = PyJPValue_getJavaSlot((PyObject*) self);
-	if (javaSlot == NULL)
+	if (javaSlot == nullptr)
 	{  // GCOVR_EXCL_START
 		// A slot is required
 		PyErr_SetString(PyExc_TypeError, "Java slot is not set on Java char");
-		return 0;
+		return nullptr;
 	}  // GCOVR_EXCL_STOP
 	if (isNull(javaSlot))
 		return JPPyString::fromStringUTF8("None").keep();
 	return PyUnicode_FromOrdinal(fromJPChar(self));
-	JP_PY_CATCH(NULL);  // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr);  // GCOVR_EXCL_LINE
 }
 
 static PyObject *PyJPChar_repr(PyJPChar *self)
@@ -242,16 +261,16 @@ static PyObject *PyJPChar_repr(PyJPChar *self)
 	JP_PY_TRY("PyJPChar_repr");
 	PyJPModule_getContext(); // Check that JVM is running
 	JPValue *javaSlot = PyJPValue_getJavaSlot((PyObject*) self);
-	if (javaSlot == NULL)
+	if (javaSlot == nullptr)
 	{  // GCOVR_EXCL_START
 		// A slot is required
 		PyErr_SetString(PyExc_TypeError, "Java slot is not set on Java char");
-		return 0;
+		return nullptr;
 	}  // GCOVR_EXCL_STOP
 	if (isNull(javaSlot))
 		return JPPyString::fromStringUTF8("None").keep();
 	return PyUnicode_Type.tp_repr((PyObject*) self);
-	JP_PY_CATCH(NULL);  // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr);  // GCOVR_EXCL_LINE
 }
 
 static PyObject *PyJPChar_index(PyJPChar *self)
@@ -260,9 +279,9 @@ static PyObject *PyJPChar_index(PyJPChar *self)
 	PyJPModule_getContext();  // Check that JVM is running
 	JPValue *javaSlot = PyJPValue_getJavaSlot((PyObject*) self);
 	if (assertNotNull(javaSlot))
-		return 0;
+		return nullptr;
 	return PyLong_FromLong(fromJPChar(self));
-	JP_PY_CATCH(NULL);  // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr);  // GCOVR_EXCL_LINE
 }
 
 static PyObject *PyJPChar_float(PyJPChar *self)
@@ -271,9 +290,9 @@ static PyObject *PyJPChar_float(PyJPChar *self)
 	PyJPModule_getContext();  // Check that JVM is running
 	JPValue *javaSlot = PyJPValue_getJavaSlot((PyObject*) self);
 	if (assertNotNull(javaSlot))
-		return 0;
+		return nullptr;
 	return PyFloat_FromDouble(fromJPChar(self));
-	JP_PY_CATCH(NULL);  // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr);  // GCOVR_EXCL_LINE
 }
 
 static PyObject *PyJPChar_abs(PyJPChar *self)
@@ -282,12 +301,12 @@ static PyObject *PyJPChar_abs(PyJPChar *self)
 	PyJPModule_getContext();  // Check that JVM is running
 	JPValue *javaSlot = PyJPValue_getJavaSlot((PyObject*) self);
 	if (assertNotNull(javaSlot))
-		return 0;
+		return nullptr;
 
 	// Promote to int as per Java rules
 	JPPyObject v = JPPyObject::call(PyLong_FromLong(fromJPChar(self)));
 	return PyLong_Type.tp_as_number->nb_absolute(v.get());
-	JP_PY_CATCH(NULL);  // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr);  // GCOVR_EXCL_LINE
 }
 
 static Py_ssize_t PyJPChar_len(PyJPChar *self)
@@ -305,28 +324,28 @@ static PyObject *apply(PyObject *first, PyObject *second, PyObject* (*func)(PyOb
 {
 	JPValue *slot0 = PyJPValue_getJavaSlot(first);
 	JPValue *slot1 = PyJPValue_getJavaSlot(second);
-	if (slot0 != NULL && slot1 != NULL)
+	if (slot0 != nullptr && slot1 != nullptr)
 	{	
 		if (assertNotNull(slot0))
-			return 0;
+			return nullptr;
 		if (assertNotNull(slot1))
-			return 0;
+			return nullptr;
 		JPPyObject v1 = JPPyObject::call(PyLong_FromLong(fromJPChar((PyJPChar*)first)));
 		JPPyObject v2 = JPPyObject::call(PyLong_FromLong(fromJPChar((PyJPChar*)second)));
 		return func(v1.get(), v2.get());
 	}
-	else if (slot0 != NULL)
+	else if (slot0 != nullptr)
 	{
 		if (assertNotNull(slot0))
-			return 0;
+			return nullptr;
 		// Promote to int as per Java rules
 		JPPyObject v = JPPyObject::call(PyLong_FromLong(fromJPChar((PyJPChar*)first)));
 		return func(v.get(), second);
 	}
-	else if (slot1 != NULL)
+	else if (slot1 != nullptr)
 	{
 		if (assertNotNull(slot1))
-			return 0;
+			return nullptr;
 		// Promote to int as per Java rules
 		JPPyObject v = JPPyObject::call(PyLong_FromLong(fromJPChar((PyJPChar*)second)));
 		return func(first, v.get());
@@ -340,7 +359,7 @@ static  PyObject *PyJPChar_and(PyObject *first, PyObject *second)
 	JP_PY_TRY("PyJPChar_and");
 	PyJPModule_getContext();  // Check that JVM is running
 	return apply(first, second, PyNumber_And);
-	JP_PY_CATCH(NULL);  // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr);  // GCOVR_EXCL_LINE
 }
 
 static  PyObject *PyJPChar_or(PyObject *first, PyObject *second)
@@ -348,7 +367,7 @@ static  PyObject *PyJPChar_or(PyObject *first, PyObject *second)
 	JP_PY_TRY("PyJPChar_or");
 	PyJPModule_getContext();  // Check that JVM is running
 	return apply(first, second, PyNumber_Or);
-	JP_PY_CATCH(NULL);  // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr);  // GCOVR_EXCL_LINE
 }
 
 static  PyObject *PyJPChar_xor(PyObject *first, PyObject *second)
@@ -356,7 +375,7 @@ static  PyObject *PyJPChar_xor(PyObject *first, PyObject *second)
 	JP_PY_TRY("PyJPChar_xor");
 	PyJPModule_getContext();  // Check that JVM is running
 	return apply(first, second, PyNumber_Xor);
-	JP_PY_CATCH(NULL);  // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr);  // GCOVR_EXCL_LINE
 }
 
 static  PyObject *PyJPChar_add(PyObject *first, PyObject *second)
@@ -365,20 +384,20 @@ static  PyObject *PyJPChar_add(PyObject *first, PyObject *second)
 	PyJPModule_getContext();  // Check that JVM is running
 	JPValue *slot0 = PyJPValue_getJavaSlot(first);
 	JPValue *slot1 = PyJPValue_getJavaSlot(second);
-	if (slot1 != NULL && slot0 != NULL)
+	if (slot1 != nullptr && slot0 != nullptr)
 	{	
 		if (assertNotNull(slot0))
-			return 0;
+			return nullptr;
 		if (assertNotNull(slot1))
-			return 0;
+			return nullptr;
 		JPPyObject v1 = JPPyObject::call(PyLong_FromLong(fromJPChar((PyJPChar*)first)));
 		JPPyObject v2 = JPPyObject::call(PyLong_FromLong(fromJPChar((PyJPChar*)second)));
 		return PyNumber_Add(v1.get(), v2.get());
 	}
-	else if (slot0 != NULL)
+	else if (slot0 != nullptr)
 	{
 		if (assertNotNull(slot0))
-			return 0;
+			return nullptr;
 		if (PyUnicode_Check(second))
 			return PyUnicode_Concat(first, second);
 
@@ -386,10 +405,10 @@ static  PyObject *PyJPChar_add(PyObject *first, PyObject *second)
 		JPPyObject v = JPPyObject::call(PyLong_FromLong(fromJPChar((PyJPChar*)first)));
 		return PyNumber_Add(v.get(), second);
 	}
-	else if (slot1 != NULL)
+	else if (slot1 != nullptr)
 	{
 		if (assertNotNull(slot1))
-			return 0;
+			return nullptr;
 		if (PyUnicode_Check(first))
 			return PyUnicode_Concat(first, second);
 
@@ -398,7 +417,7 @@ static  PyObject *PyJPChar_add(PyObject *first, PyObject *second)
 		return PyNumber_Add(first, v.get());
 	}
 	return notSupported();
-	JP_PY_CATCH(NULL);  // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr);  // GCOVR_EXCL_LINE
 }
 
 
@@ -407,7 +426,7 @@ static  PyObject *PyJPChar_subtract(PyObject *first, PyObject *second)
 	JP_PY_TRY("PyJPChar_subtract");
 	PyJPModule_getContext();  // Check that JVM is running
 	return apply(first, second, PyNumber_Subtract);
-	JP_PY_CATCH(NULL);  // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr);  // GCOVR_EXCL_LINE
 }
 
 static  PyObject *PyJPChar_mult(PyObject *first, PyObject *second)
@@ -415,7 +434,7 @@ static  PyObject *PyJPChar_mult(PyObject *first, PyObject *second)
 	JP_PY_TRY("PyJPChar_mult");
 	PyJPModule_getContext();  // Check that JVM is running
 	return apply(first, second, PyNumber_Multiply);
-	JP_PY_CATCH(NULL);  // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr);  // GCOVR_EXCL_LINE
 }
 
 static  PyObject *PyJPChar_rshift(PyObject *first, PyObject *second)
@@ -423,7 +442,7 @@ static  PyObject *PyJPChar_rshift(PyObject *first, PyObject *second)
 	JP_PY_TRY("PyJPChar_rshift");
 	PyJPModule_getContext();  // Check that JVM is running
 	return apply(first, second, PyNumber_Rshift);
-	JP_PY_CATCH(NULL);  // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr);  // GCOVR_EXCL_LINE
 }
 
 static  PyObject *PyJPChar_lshift(PyObject *first, PyObject *second)
@@ -431,7 +450,7 @@ static  PyObject *PyJPChar_lshift(PyObject *first, PyObject *second)
 	JP_PY_TRY("PyJPChar_lshift");
 	PyJPModule_getContext();  // Check that JVM is running
 	return apply(first, second, PyNumber_Lshift);
-	JP_PY_CATCH(NULL);  // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr);  // GCOVR_EXCL_LINE
 }
 
 static  PyObject *PyJPChar_floordiv(PyObject *first, PyObject *second)
@@ -439,7 +458,7 @@ static  PyObject *PyJPChar_floordiv(PyObject *first, PyObject *second)
 	JP_PY_TRY("PyJPChar_floordiv");
 	PyJPModule_getContext();  // Check that JVM is running
 	return apply(first, second, PyNumber_FloorDivide);
-	JP_PY_CATCH(NULL);  // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr);  // GCOVR_EXCL_LINE
 }
 
 static  PyObject *PyJPChar_divmod(PyObject *first, PyObject *second)
@@ -447,7 +466,7 @@ static  PyObject *PyJPChar_divmod(PyObject *first, PyObject *second)
 	JP_PY_TRY("PyJPChar_divmod");
 	PyJPModule_getContext();  // Check that JVM is running
 	return apply(first, second, PyNumber_Divmod);
-	JP_PY_CATCH(NULL);  // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr);  // GCOVR_EXCL_LINE
 }
 
 static PyObject *PyJPChar_neg(PyJPChar *self)
@@ -456,11 +475,11 @@ static PyObject *PyJPChar_neg(PyJPChar *self)
 	PyJPModule_getContext();  // Check that JVM is running
 	JPValue *javaSlot = PyJPValue_getJavaSlot((PyObject*) self);
 	if (assertNotNull(javaSlot))
-		return 0;
+		return nullptr;
 	// Promote to int as per Java rules
 	JPPyObject v = JPPyObject::call(PyLong_FromLong(fromJPChar(self)));
 	return PyNumber_Negative(v.get());
-	JP_PY_CATCH(0);  // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr);  // GCOVR_EXCL_LINE
 }
 
 static PyObject *PyJPChar_pos(PyJPChar *self)
@@ -469,11 +488,11 @@ static PyObject *PyJPChar_pos(PyJPChar *self)
 	PyJPModule_getContext();  // Check that JVM is running
 	JPValue *javaSlot = PyJPValue_getJavaSlot((PyObject*) self);
 	if (assertNotNull(javaSlot))
-		return 0;
+		return nullptr;
 	// Promote to int as per Java rules
 	JPPyObject v = JPPyObject::call(PyLong_FromLong(fromJPChar(self)));
 	return PyNumber_Positive(v.get());
-	JP_PY_CATCH(0);  // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr);  // GCOVR_EXCL_LINE
 }
 
 static PyObject *PyJPChar_inv(PyJPChar *self)
@@ -482,11 +501,11 @@ static PyObject *PyJPChar_inv(PyJPChar *self)
 	PyJPModule_getContext();  // Check that JVM is running
 	JPValue *javaSlot = PyJPValue_getJavaSlot((PyObject*) self);
 	if (assertNotNull(javaSlot))
-		return 0;
+		return nullptr;
 	// Promote to int as per Java rules
 	JPPyObject v = JPPyObject::call(PyLong_FromLong(fromJPChar(self)));
 	return PyNumber_Invert(v.get());
-	JP_PY_CATCH(0);  // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr);  // GCOVR_EXCL_LINE
 }
 
 static PyObject *PyJPJChar_compare(PyObject *self, PyObject *other, int op)
@@ -497,7 +516,7 @@ static PyObject *PyJPJChar_compare(PyObject *self, PyObject *other, int op)
 	JPValue *javaSlot0 = PyJPValue_getJavaSlot(self);
 	if (isNull(javaSlot0))
 	{
-		if (javaSlot1 != NULL && isNull(javaSlot1))
+		if (javaSlot1 != nullptr && isNull(javaSlot1))
 			other = Py_None;
 		if (op == Py_EQ)
 			return PyBool_FromLong(other == Py_None );
@@ -508,7 +527,7 @@ static PyObject *PyJPJChar_compare(PyObject *self, PyObject *other, int op)
 		return out;
 		JP_RAISE_PYTHON();
 	}
-	if (javaSlot1 != NULL && isNull(javaSlot1))
+	if (javaSlot1 != nullptr && isNull(javaSlot1))
 		return PyBool_FromLong(op == Py_NE);
 
 	if (PyUnicode_Check(other))
@@ -531,7 +550,7 @@ static PyObject *PyJPJChar_compare(PyObject *self, PyObject *other, int op)
 		JPPyObject v = JPPyObject::call(PyLong_FromLong(fromJPChar((PyJPChar*) self)));
 		return PyLong_Type.tp_richcompare(v.get(), other, op);
 	}
-	if (javaSlot1 != NULL)
+	if (javaSlot1 != nullptr)
 	{
 		// char  <=> object
 		// object <=> char
@@ -551,7 +570,7 @@ static PyObject *PyJPJChar_compare(PyObject *self, PyObject *other, int op)
 	PyObject *out = Py_NotImplemented;
 	Py_INCREF(out);
 	return out;
-	JP_PY_CATCH(NULL);  // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr);  // GCOVR_EXCL_LINE
 }
 
 static Py_hash_t PyJPChar_hash(PyObject *self)
@@ -579,12 +598,12 @@ static int PyJPChar_bool(PyJPChar *self)
 
 static PyMethodDef charMethods[] = {
 	//	{"thing", (PyCFunction) PyJPMethod_matchReport, METH_VARARGS, ""},
-	{NULL},
+	{nullptr},
 };
 
 struct PyGetSetDef charGetSet[] = {
 	//	{"thing", (getter) PyJPMethod_getSelf, NULL, NULL, NULL},
-	{NULL},
+	{nullptr},
 };
 
 static PyType_Slot charSlots[] = {
