@@ -59,19 +59,19 @@ public:
 } ;
 
 
-PyObject* _JArray = NULL;
-PyObject* _JChar = NULL;
-PyObject* _JObject = NULL;
-PyObject* _JInterface = NULL;
-PyObject* _JException = NULL;
-PyObject* _JClassPre = NULL;
-PyObject* _JClassPost = NULL;
-PyObject* _JClassDoc = NULL;
-PyObject* _JMethodDoc = NULL;
-PyObject* _JMethodAnnotations = NULL;
-PyObject* _JMethodCode = NULL;
-PyObject* _JObjectKey = NULL;
-PyObject* _JVMNotRunning = NULL;
+PyObject* _JArray = nullptr;
+PyObject* _JChar = nullptr;
+PyObject* _JObject = nullptr;
+PyObject* _JInterface = nullptr;
+PyObject* _JException = nullptr;
+PyObject* _JClassPre = nullptr;
+PyObject* _JClassPost = nullptr;
+PyObject* _JClassDoc = nullptr;
+PyObject* _JMethodDoc = nullptr;
+PyObject* _JMethodAnnotations = nullptr;
+PyObject* _JMethodCode = nullptr;
+PyObject* _JObjectKey = nullptr;
+PyObject* _JVMNotRunning = nullptr;
 
 void PyJPModule_loadResources(PyObject* module)
 {
@@ -121,7 +121,7 @@ void PyJPModule_loadResources(PyObject* module)
 		JP_PY_CHECK();
 		Py_INCREF(_JMethodCode);
 
-		_JObjectKey = PyCapsule_New(module, "constructor key", NULL);
+		_JObjectKey = PyCapsule_New(module, "constructor key", nullptr);
 
 	}	catch (JPypeException&)  // GCOVR_EXCL_LINE
 	{
@@ -148,7 +148,7 @@ void PyJP_SetStringWithCause(PyObject *exception,
 	PyObject *exc1, *val1, *tb1;
 	PyErr_Fetch(&exc1, &val1, &tb1);
 	PyErr_NormalizeException(&exc1, &val1, &tb1);
-	if (tb1 != NULL)
+	if (tb1 != nullptr)
 	{
 		PyException_SetTraceback(val1, tb1);
 		Py_DECREF(tb1);
@@ -166,14 +166,24 @@ void PyJP_SetStringWithCause(PyObject *exception,
 PyObject* PyJP_GetAttrDescriptor(PyTypeObject *type, PyObject *attr_name)
 {
 	JP_PY_TRY("Py_GetAttrDescriptor");
-	if (type->tp_mro == NULL)
-		return NULL;  // GCOVR_EXCL_LINE
+	if (type->tp_mro == nullptr)
+		return nullptr;  // GCOVR_EXCL_LINE
 
+	// Grab the mro
 	PyObject *mro = type->tp_mro;
+
+	// mro should be a tuple
 	Py_ssize_t n = PyTuple_Size(mro);
+
+	// Search the tuple for the attribute
 	for (Py_ssize_t i = 0; i < n; ++i)
 	{
-		PyTypeObject *type2 = (PyTypeObject*) PyTuple_GetItem(mro, i);
+		auto *type2 = (PyTypeObject*) PyTuple_GetItem(mro, i);
+		
+		// Skip objects without a functioning dictionary
+		if (type2->tp_dict == NULL)
+			continue;
+
 		PyObject *res = PyDict_GetItem(type2->tp_dict, attr_name);
 		if (res)
 		{
@@ -192,13 +202,13 @@ PyObject* PyJP_GetAttrDescriptor(PyTypeObject *type, PyObject *attr_name)
 		}
 	}
 
-	return NULL;
-	JP_PY_CATCH(NULL); // GCOVR_EXCL_LINE
+	return nullptr;
+	JP_PY_CATCH(nullptr); // GCOVR_EXCL_LINE
 }
 
 int PyJP_IsSubClassSingle(PyTypeObject* type, PyTypeObject* obj)
 {
-	if (type == NULL || obj == NULL)
+	if (type == nullptr || obj == nullptr)
 		return 0;  // GCOVR_EXCL_LINE
 	PyObject* mro1 = obj->tp_mro;
 	Py_ssize_t n1 = PyTuple_Size(mro1);
@@ -210,7 +220,7 @@ int PyJP_IsSubClassSingle(PyTypeObject* type, PyTypeObject* obj)
 
 int PyJP_IsInstanceSingle(PyObject* obj, PyTypeObject* type)
 {
-	if (type == NULL || obj == NULL)
+	if (type == nullptr || obj == nullptr)
 		return 0; // GCOVR_EXCL_LINE
 	return PyJP_IsSubClassSingle(type, Py_TYPE(obj));
 }
@@ -230,12 +240,12 @@ static PyObject* PyJPModule_startup(PyObject* module, PyObject* pyargs)
 
 	if (!PyArg_ParseTuple(pyargs, "OO!bbb", &vmPath, &PyTuple_Type, &vmOpt,
 			&ignoreUnrecognized, &convertStrings, &interrupt))
-		return NULL;
+		return nullptr;
 
 	if (!(JPPyString::check(vmPath)))
 	{
 		PyErr_SetString(PyExc_TypeError, "Java JVM path must be a string");
-		return NULL;
+		return nullptr;
 	}
 
 	string cVmPath = JPPyString::asStringUTF8(vmPath);
@@ -253,11 +263,11 @@ static PyObject* PyJPModule_startup(PyObject* module, PyObject* pyargs)
 			// TODO support unicode
 			string v = JPPyString::asStringUTF8(obj.get());
 			JP_TRACE("arg", v);
-			args.push_back(v);
+			args.emplace_back(v);
 		} else
 		{
 			PyErr_SetString(PyExc_TypeError, "VM Arguments must be strings");
-			return NULL;
+			return nullptr;
 		}
 	}
 
@@ -265,7 +275,7 @@ static PyObject* PyJPModule_startup(PyObject* module, PyObject* pyargs)
 	if (JPContext_global->isRunning())
 	{
 		PyErr_SetString(PyExc_OSError, "JVM is already started");
-		return NULL;
+		return nullptr;
 	}
 
 	// install the gc hook
@@ -274,7 +284,7 @@ static PyObject* PyJPModule_startup(PyObject* module, PyObject* pyargs)
 	JPContext_global->startJVM(cVmPath, args, ignoreUnrecognized != 0, convertStrings != 0, interrupt != 0);
 
 	Py_RETURN_NONE;
-	JP_PY_CATCH(NULL);
+	JP_PY_CATCH(nullptr);
 }
 
 static PyObject* PyJPModule_shutdown(PyObject* obj, PyObject* pyargs, PyObject* kwargs)
@@ -284,11 +294,11 @@ static PyObject* PyJPModule_shutdown(PyObject* obj, PyObject* pyargs, PyObject*
 	char freeJVM = true;
 
 	if (!PyArg_ParseTuple(pyargs, "bb", &destroyJVM, &freeJVM))
-		return NULL;
+		return nullptr;
 
 	JPContext_global->shutdownJVM(destroyJVM, freeJVM);
 	Py_RETURN_NONE;
-	JP_PY_CATCH(NULL);
+	JP_PY_CATCH(nullptr);
 }
 #endif
 
@@ -304,7 +314,7 @@ static PyObject* PyJPModule_attachThread(PyObject* obj)
 	JP_PY_TRY("PyJPModule_attachThread");
 	PyJPModule_getContext()->attachCurrentThread();
 	Py_RETURN_NONE;
-	JP_PY_CATCH(NULL);
+	JP_PY_CATCH(nullptr);
 }
 
 static PyObject* PyJPModule_attachThreadAsDaemon(PyObject* obj)
@@ -312,7 +322,7 @@ static PyObject* PyJPModule_attachThreadAsDaemon(PyObject* obj)
 	JP_PY_TRY("PyJPModule_attachThreadAsDaemon");
 	PyJPModule_getContext()->attachCurrentThreadAsDaemon();
 	Py_RETURN_NONE;
-	JP_PY_CATCH(NULL);
+	JP_PY_CATCH(nullptr);
 }
 
 static PyObject* PyJPModule_detachThread(PyObject* obj)
@@ -321,7 +331,7 @@ static PyObject* PyJPModule_detachThread(PyObject* obj)
 	if (JPContext_global->isRunning())
 		JPContext_global->detachCurrentThread();
 	Py_RETURN_NONE;
-	JP_PY_CATCH(NULL);
+	JP_PY_CATCH(nullptr);
 }
 #endif
 
@@ -331,14 +341,14 @@ static PyObject* PyJPModule_isThreadAttached(PyObject* obj)
 	if (!JPContext_global->isRunning())
 		return PyBool_FromLong(0); // GCOVR_EXCL_LINE
 	return PyBool_FromLong(JPContext_global->isThreadAttached());
-	JP_PY_CATCH(NULL);
+	JP_PY_CATCH(nullptr);
 }
 
 // Cleanup hook for Py_buffer
 
 static void releaseView(void* view)
 {
-	if (view != 0)
+	if (view != nullptr)
 	{
 		PyBuffer_Release((Py_buffer*) view);
 		delete (Py_buffer*) view;
@@ -355,7 +365,7 @@ static PyObject* PyJPModule_convertToDirectByteBuffer(PyObject* self, PyObject*
 	{
 		JPViewWrapper vw;
 		if (PyObject_GetBuffer(src, vw.view, PyBUF_WRITABLE) == -1)
-			return NULL;
+			return nullptr;
 
 		// Create a byte buffer
 		jvalue v;
@@ -363,12 +373,12 @@ static PyObject* PyJPModule_convertToDirectByteBuffer(PyObject* self, PyObject*
 
 		// Bind lifespan of the view to the java object.
 		frame.registerRef(v.l, vw.view, &releaseView);
-		vw.view = 0;
+		vw.view = nullptr;
 		JPClass *type = frame.findClassForObject(v.l);
 		return type->convertToPythonObject(frame, v, false).keep();
 	}
 	PyErr_SetString(PyExc_TypeError, "convertToDirectByteBuffer requires buffer support");
-	JP_PY_CATCH(NULL);
+	JP_PY_CATCH(nullptr);
 }
 
 static PyObject* PyJPModule_enableStacktraces(PyObject* self, PyObject* src)
@@ -385,23 +395,23 @@ PyObject *PyJPModule_newArrayType(PyObject *module, PyObject *args)
 
 	PyObject *type, *dims;
 	if (!PyArg_ParseTuple(args, "OO", &type, &dims))
-		return NULL;
+		return nullptr;
 	if (!PyIndex_Check(dims))
 	{
 		PyErr_SetString(PyExc_TypeError, "dims must be an integer");
-		return NULL;
+		return nullptr;
 	}
 	long d = PyLong_AsLong(dims);
 	JPClass* cls = PyJPClass_getJPClass(type);
-	if (cls == NULL)
+	if (cls == nullptr)
 	{
 		PyErr_SetString(PyExc_TypeError, "Java class required");
-		return NULL;
+		return nullptr;
 	}
 
 	JPClass* arraycls = cls->newArrayType(frame, d);
 	return PyJPClass_create(frame, arraycls).keep();
-	JP_PY_CATCH(NULL);
+	JP_PY_CATCH(nullptr);
 }
 
 PyObject *PyJPModule_getClass(PyObject* module, PyObject *obj)
@@ -415,30 +425,30 @@ PyObject *PyJPModule_getClass(PyObject* module, PyObject *obj)
 	{
 		// String From Python
 		cls = frame.findClassByName(JPPyString::asStringUTF8(obj));
-		if (cls == NULL)
+		if (cls == nullptr)
 		{
 			PyErr_SetString(PyExc_ValueError, "Unable to find Java class");
-			return NULL;
+			return nullptr;
 		}
 	} else
 	{
 		// From an existing java.lang.Class object
 		JPValue *value = PyJPValue_getJavaSlot(obj);
-		if (value == 0 || value->getClass() != context->_java_lang_Class)
+		if (value == nullptr || value->getClass() != context->_java_lang_Class)
 		{
 			PyErr_Format(PyExc_TypeError, "JClass requires str or java.lang.Class instance, not '%s'", Py_TYPE(obj)->tp_name);
-			return NULL;
+			return nullptr;
 		}
 		cls = frame.findClass((jclass) value->getValue().l);
-		if (cls == NULL)
+		if (cls == nullptr)
 		{
 			PyErr_SetString(PyExc_ValueError, "Unable to find class");
-			return NULL;
+			return nullptr;
 		}
 	}
 
 	return PyJPClass_create(frame, cls).keep();
-	JP_PY_CATCH(NULL);
+	JP_PY_CATCH(nullptr);
 }
 
 PyObject *PyJPModule_hasClass(PyObject* module, PyObject *obj)
@@ -454,33 +464,33 @@ PyObject *PyJPModule_hasClass(PyObject* module, PyObject *obj)
 	{
 		// String From Python
 		cls = frame.findClassByName(JPPyString::asStringUTF8(obj));
-		if (cls == NULL)
+		if (cls == nullptr)
 		{
 			PyErr_SetString(PyExc_ValueError, "Unable to find Java class");
-			return NULL;
+			return nullptr;
 		}
 	} else
 	{
 		PyErr_Format(PyExc_TypeError, "str is required, not '%s'", Py_TYPE(obj)->tp_name);
-		return NULL;
+		return nullptr;
 	}
 
-	PyObject *host = (PyObject*) cls->getHost();
-	return PyBool_FromLong(host != NULL);
-	JP_PY_CATCH(NULL);
+	auto *host = (PyObject*) cls->getHost();
+	return PyBool_FromLong(host != nullptr);
+	JP_PY_CATCH(nullptr);
 }
 
 static PyObject *PyJPModule_arrayFromBuffer(PyObject *module, PyObject *args, PyObject *kwargs)
 {
 	JP_PY_TRY("PyJPModule_arrayFromBuffer");
-	PyObject *source = 0;
-	PyObject *dtype = 0;
+	PyObject *source = nullptr;
+	PyObject *dtype = nullptr;
 	if (!PyArg_ParseTuple(args, "OO", &source, &dtype))
-		return NULL;
+		return nullptr;
 	if (!PyObject_CheckBuffer(source))
 	{
 		PyErr_Format(PyExc_TypeError, "'%s' does not support buffers", Py_TYPE(source)->tp_name);
-		return NULL;
+		return nullptr;
 	}
 
 	// NUMPy does a series of probes looking for the best supported format,
@@ -501,8 +511,8 @@ static PyObject *PyJPModule_arrayFromBuffer(PyObject *module, PyObject *args, Py
 			return PyJPModule_convertBuffer(buffer, dtype);
 	}
 	PyErr_Format(PyExc_TypeError, "buffer protocol for '%s' not supported", Py_TYPE(source)->tp_name);
-	return NULL;
-	JP_PY_CATCH(NULL);
+	return nullptr;
+	JP_PY_CATCH(nullptr);
 }
 
 PyObject *PyJPModule_collect(PyObject* module, PyObject *obj)
@@ -514,7 +524,7 @@ PyObject *PyJPModule_collect(PyObject* module, PyObject *obj)
 	if (!PyUnicode_Check(a1))
 	{
 		PyErr_SetString(PyExc_TypeError, "Bad callback argument");
-		return NULL;
+		return nullptr;
 	}
 	if (PyUnicode_ReadChar(a1, 2) == 'a')
 	{
@@ -535,17 +545,17 @@ PyObject *PyJPModule_gcStats(PyObject* module, PyObject *obj)
 	context->m_GC->getStats(stats);
 	PyObject *out = PyDict_New();
 	PyObject *res;
-	PyDict_SetItemString(out, "current", res = PyLong_FromSsize_t(stats.current_rss));
+	PyDict_SetItemString(out, "current", res = PyLong_FromSsize_t((Py_ssize_t)(stats.current_rss)));
 	Py_DECREF(res);
-	PyDict_SetItemString(out, "java", res = PyLong_FromSsize_t(stats.java_rss));
+	PyDict_SetItemString(out, "java", res = PyLong_FromSsize_t((Py_ssize_t)(stats.java_rss)));
 	Py_DECREF(res);
-	PyDict_SetItemString(out, "python", res = PyLong_FromSsize_t(stats.python_rss));
+	PyDict_SetItemString(out, "python", res = PyLong_FromSsize_t((Py_ssize_t)(stats.python_rss)));
 	Py_DECREF(res);
-	PyDict_SetItemString(out, "max", res = PyLong_FromSsize_t(stats.max_rss));
+	PyDict_SetItemString(out, "max", res = PyLong_FromSsize_t((Py_ssize_t)(stats.max_rss)));
 	Py_DECREF(res);
-	PyDict_SetItemString(out, "min", res = PyLong_FromSsize_t(stats.min_rss));
+	PyDict_SetItemString(out, "min", res = PyLong_FromSsize_t((Py_ssize_t)(stats.min_rss)));
 	Py_DECREF(res);
-	PyDict_SetItemString(out, "triggered", res = PyLong_FromSsize_t(stats.python_triggered));
+	PyDict_SetItemString(out, "triggered", res = PyLong_FromSsize_t((Py_ssize_t)(stats.python_triggered)));
 	Py_DECREF(res);
 	return out;
 }
@@ -557,12 +567,12 @@ static PyObject* PyJPModule_isPackage(PyObject *module, PyObject *pkg)
 	if (!PyUnicode_Check(pkg))
 	{
 		PyErr_Format(PyExc_TypeError, "isPackage required unicode");
-		return NULL;
+		return nullptr;
 	}
 	JPContext *context = PyJPModule_getContext();
 	JPJavaFrame frame = JPJavaFrame::outer(context);
 	return PyBool_FromLong(frame.isPackage(JPPyString::asStringUTF8(pkg)));
-	JP_PY_CATCH(NULL); // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr); // GCOVR_EXCL_LINE
 }
 
 
@@ -610,7 +620,7 @@ PyObject* examine(PyObject *module, PyObject *other)
 	printf("======\n");
 
 	return PyBool_FromLong(ret);
-	JP_PY_CATCH(NULL);
+	JP_PY_CATCH(nullptr);
 }
 // GCOVR_EXCL_STOP
 #endif
@@ -695,7 +705,7 @@ static PyMethodDef moduleMethods[] = {
 	{"examine", (PyCFunction) examine, METH_O, ""},
 
 	// sentinel
-	{NULL}
+	{nullptr}
 };
 
 static struct PyModuleDef moduledef = {
@@ -706,8 +716,8 @@ static struct PyModuleDef moduledef = {
 	moduleMethods,
 };
 
-PyObject *PyJPModule = NULL;
-JPContext* JPContext_global = NULL;
+PyObject *PyJPModule = nullptr;
+JPContext* JPContext_global = nullptr;
 
 PyMODINIT_FUNC PyInit__jpype()
 {
@@ -719,7 +729,7 @@ PyMODINIT_FUNC PyInit__jpype()
 	// PyJPModule = module;
 	Py_INCREF(module);
 	PyJPModule = module;
-	PyModule_AddStringConstant(module, "__version__", "1.4.1");
+	PyModule_AddStringConstant(module, "__version__", "1.5.0");
 	
 	// Our module will be used for PyFrame object and it is a requirement that
 	// we have a builtins in our dictionary.
@@ -746,7 +756,7 @@ PyMODINIT_FUNC PyInit__jpype()
 	_PyJPModule_trace = true;
 
 	return module;
-	JP_PY_CATCH(NULL); // GCOVR_EXCL_LINE
+	JP_PY_CATCH(nullptr); // GCOVR_EXCL_LINE
 }
 
 #ifdef __cplusplus
@@ -785,18 +795,18 @@ static PyObject *PyJPModule_convertBuffer(JPPyBuffer& buffer, PyObject *dtype)
 
 	// Okay two possibilities here.  We have a valid dtype specified,
 	// or we need to figure it out from the buffer.
-	JPClass *cls = NULL;
+	JPClass *cls = nullptr;
 
-	if (view.suboffsets != NULL && view.suboffsets[view.ndim - 1] > 0)
+	if (view.suboffsets != nullptr && view.suboffsets[view.ndim - 1] > 0)
 	{
 		PyErr_Format(PyExc_TypeError, "last dimension is not contiguous");
-		return NULL;
+		return nullptr;
 	}
 
 	// First lets find out what we are unpacking
 	Py_ssize_t itemsize = view.itemsize;
 	char *format = view.format;
-	if (format == NULL)
+	if (format == nullptr)
 		format = "B";
 	// Standard size for 'l' is 4 in docs, but numpy uses format 'l' for long long
 	if (itemsize == 8 && format[0] == 'l')
@@ -804,13 +814,13 @@ static PyObject *PyJPModule_convertBuffer(JPPyBuffer& buffer, PyObject *dtype)
 	if (itemsize == 8 && format[0] == 'L')
 		format = "Q";
 
-	if (dtype != NULL && dtype != Py_None )
+	if (dtype != nullptr && dtype != Py_None )
 	{
 		cls = PyJPClass_getJPClass(dtype);
-		if (cls == NULL  || !cls->isPrimitive())
+		if (cls == nullptr  || !cls->isPrimitive())
 		{
 			PyErr_Format(PyExc_TypeError, "'%s' is not a Java primitive type", Py_TYPE(dtype)->tp_name);
-			return NULL;
+			return nullptr;
 		}
 	} else
 	{
@@ -841,22 +851,22 @@ static PyObject *PyJPModule_convertBuffer(JPPyBuffer& buffer, PyObject *dtype)
 			default:
 				break;
 		}
-		if (cls == NULL)
+		if (cls == nullptr)
 		{
 			PyErr_Format(PyExc_TypeError, "'%s' type code not supported without dtype specified", format);
-			return NULL;
+			return nullptr;
 		}
 	}
 
 	// Now we have a valid format code, so next lets get a converter for
 	// the type.
-	JPPrimitiveType *pcls = (JPPrimitiveType *) cls;
+	auto *pcls = dynamic_cast<JPPrimitiveType *>( cls);
 
 	// Convert the shape
 	Py_ssize_t subs = 1;
 	Py_ssize_t base = 1;
-	jintArray jdims = (jintArray) context->_int->newArrayOf(frame, view.ndim);
-	if (view.shape != NULL)
+	auto jdims = (jintArray) context->_int->newArrayOf(frame, view.ndim);
+	if (view.shape != nullptr)
 	{
 		JPPrimitiveArrayAccessor<jintArray, jint*> accessor(frame, jdims,
 				&JPJavaFrame::GetIntArrayElements, &JPJavaFrame::ReleaseIntArrayElements);
@@ -876,7 +886,7 @@ static PyObject *PyJPModule_convertBuffer(JPPyBuffer& buffer, PyObject *dtype)
 		if (view.ndim > 1)
 		{
 			PyErr_Format(PyExc_TypeError, "buffer dims inconsistent");
-			return NULL;
+			return nullptr;
 		}
 		base = view.len / view.itemsize;
 	}
