@@ -15,12 +15,11 @@
 #   See NOTICE file for details.
 #
 # *****************************************************************************
+
 from . import utils
 from . import dist
 from . import platform
 from . import build_ext
-from . import build_java
-from . import build_thunk
 from . import test_java
 from . import sdist
 from . import pytester
