@@ -22,7 +22,7 @@ import jpype
 import logging
 from os import path
 import sys
-import unittest
+import unittest  # Extensively used as common.unittest.
 
 CLASSPATH = None
 fast = False
@@ -127,8 +127,6 @@ class JPypeTestCase(unittest.TestCase):
             jpype.startJVM(jvm_path, *args,
                            convertStrings=self._convertStrings)
         self.jpype = jpype.JPackage('jpype')
-        if sys.version < '3':
-            self.assertCountEqual = self.assertItemsEqual
 
     def tearDown(self):
         pass
