@@ -1,12 +1,10 @@
 # This file is Public Domain and may be used without restrictions,
-# because noone should have to waste their lives typing this again.
-import _jpype
+# because nobody should have to waste their lives typing this again.
 import jpype
 from jpype.types import *
 from jpype import java
 import jpype.dbapi2 as dbapi2
 import common
-import time
 import datetime
 import decimal
 import threading
@@ -16,7 +14,7 @@ java = jpype.java
 try:
     import zlib
 except ImportError:
-    zlib = None
+    zlib = None  # type: ignore[assignment]
 
 
 db_name = "jdbc:h2:mem:testdb"
@@ -795,6 +793,7 @@ class CursorTestCase(common.JPypeTestCase):
                 cu.execute("insert into booze(name,price) values(?,?)", object())
 
 
+@common.unittest.skipUnless(zlib, "requires zlib")
 class AdapterTestCase(common.JPypeTestCase):
     def setUp(self):
         common.JPypeTestCase.setUp(self)
@@ -907,6 +906,7 @@ class ConverterTestCase(common.JPypeTestCase):
                 f = cu.fetchone(types=[])
 
 
+@common.unittest.skipUnless(zlib, "requires zlib")
 class GettersTestCase(common.JPypeTestCase):
     def setUp(self):
         common.JPypeTestCase.setUp(self)
