@@ -16,7 +16,7 @@ java = jpype.java
 try:
     import zlib
 except ImportError:
-    zlib = None
+    zlib = None  # type: ignore[assignment]
 
 
 db_name = "jdbc:hsqldb:mem:myDb"
@@ -24,6 +24,7 @@ db_name = "jdbc:hsqldb:mem:myDb"
 #first = "jdbc:derby:memory:myDb;create=True"
 
 
+@common.unittest.skipUnless(zlib, "requires zlib")
 class ConnectTestCase(common.JPypeTestCase):
     def setUp(self):
         common.JPypeTestCase.setUp(self)
@@ -793,6 +794,7 @@ class CursorTestCase(common.JPypeTestCase):
                 cu.execute("insert into booze(name,price) values(?,?)", object())
 
 
+@common.unittest.skipUnless(zlib, "requires zlib")
 class AdapterTestCase(common.JPypeTestCase):
     def setUp(self):
         common.JPypeTestCase.setUp(self)
@@ -905,6 +907,7 @@ class ConverterTestCase(common.JPypeTestCase):
                 f = cu.fetchone(types=[])
 
 
+@common.unittest.skipUnless(zlib, "requires zlib")
 class GettersTestCase(common.JPypeTestCase):
     def setUp(self):
         common.JPypeTestCase.setUp(self)
@@ -921,7 +924,7 @@ class GettersTestCase(common.JPypeTestCase):
                 pass
 
 
-@common.unittest.skip
+@common.unittest.skip  # type: ignore
 class TransactionsTestCase(common.JPypeTestCase):
     def setUp(self):
         common.JPypeTestCase.setUp(self)
